#[repr(transparent)]pub struct Bound<'py, T>(Python<'py>, ManuallyDrop<Py<T>>);Expand description
A Python thread-attached equivalent to Py<T>.
This type can be thought of as equivalent to the tuple (Py<T>, Python<'py>). By having the 'py
lifetime of the Python<'py> token, this ties the lifetime of the Bound<'py, T> smart pointer
to the lifetime the thread is attached to the Python interpreter and allows PyO3 to call Python APIs
at maximum efficiency.
To access the object in situations where the thread is not attached, convert it to Py<T>
using .unbind(). This includes, for example, usage in
Python::detach’s closure.
See the guide for more detail.
Tuple Fields
0: Python<'py>1: ManuallyDrop<Py<T>>Implementations
Sourceimpl Bound<'_, PyType>
impl Bound<'_, PyType>
pub(crate) fn get_slot<const S: c_int>(
&self,
slot: Slot<S>,
) -> <Slot<S> as GetSlotImpl>::Typewhere
Slot<S>: GetSlotImpl,
Sourceimpl<'py, T> Bound<'py, T>where
T: PyClass,
impl<'py, T> Bound<'py, T>where
T: PyClass,
Sourcepub fn new(
py: Python<'py>,
value: impl Into<PyClassInitializer<T>>,
) -> PyResult<Bound<'py, T>>
pub fn new( py: Python<'py>, value: impl Into<PyClassInitializer<T>>, ) -> PyResult<Bound<'py, T>>
Creates a new instance Bound<T> of a #[pyclass] on the Python heap.
Examples
use pyo3::prelude::*;
#[pyclass]
struct Foo {/* fields omitted */}
let foo: Py<Foo> = Python::attach(|py| -> PyResult<_> {
let foo: Bound<'_, Foo> = Bound::new(py, Foo {})?;
Ok(foo.into())
})?;Sourceimpl<'py, T> Bound<'py, T>
impl<'py, T> Bound<'py, T>
Sourcepub fn cast<U>(&self) -> Result<&Bound<'py, U>, CastError<'_, 'py>>where
U: PyTypeCheck,
pub fn cast<U>(&self) -> Result<&Bound<'py, U>, CastError<'_, 'py>>where
U: PyTypeCheck,
Cast this to a concrete Python type or pyclass.
Note that you can often avoid casting yourself by just specifying the desired type in function or method signatures. However, manual casting is sometimes necessary.
For extracting a Rust-only type, see extract.
This performs a runtime type check using the equivalent of Python’s
isinstance(self, U).
Example: Casting to a specific Python object
use pyo3::prelude::*;
use pyo3::types::{PyDict, PyList};
Python::attach(|py| {
let dict = PyDict::new(py);
assert!(dict.is_instance_of::<PyAny>());
let any = dict.as_any();
assert!(any.cast::<PyDict>().is_ok());
assert!(any.cast::<PyList>().is_err());
});Example: Getting a reference to a pyclass
This is useful if you want to mutate a Py<PyAny> that might actually be a pyclass.
use pyo3::prelude::*;
#[pyclass]
struct Class {
i: i32,
}
Python::attach(|py| {
let class = Bound::new(py, Class { i: 0 })?.into_any();
let class_bound: &Bound<'_, Class> = class.cast()?;
class_bound.borrow_mut().i += 1;
// Alternatively you can get a `PyRefMut` directly
let class_ref: PyRefMut<'_, Class> = class.extract()?;
assert_eq!(class_ref.i, 1);
Ok(())
})Sourcepub fn cast_into<U>(self) -> Result<Bound<'py, U>, CastIntoError<'py>>where
U: PyTypeCheck,
pub fn cast_into<U>(self) -> Result<Bound<'py, U>, CastIntoError<'py>>where
U: PyTypeCheck,
Like cast but takes ownership of self.
In case of an error, it is possible to retrieve self again via
CastIntoError::into_inner.
Example
use pyo3::prelude::*;
use pyo3::types::{PyDict, PyList};
Python::attach(|py| {
let obj: Bound<'_, PyAny> = PyDict::new(py).into_any();
let obj: Bound<'_, PyAny> = match obj.cast_into::<PyList>() {
Ok(_) => panic!("obj should not be a list"),
Err(err) => err.into_inner(),
};
// obj is a dictionary
assert!(obj.cast_into::<PyDict>().is_ok());
})Sourcepub fn cast_exact<U>(&self) -> Result<&Bound<'py, U>, CastError<'_, 'py>>where
U: PyTypeInfo,
pub fn cast_exact<U>(&self) -> Result<&Bound<'py, U>, CastError<'_, 'py>>where
U: PyTypeInfo,
Cast this to a concrete Python type or pyclass (but not a subclass of it).
It is almost always better to use cast because it accounts for Python
subtyping. Use this method only when you do not want to allow subtypes.
The advantage of this method over cast is that it is faster. The
implementation of cast_exact uses the equivalent of the Python expression type(self) is U, whereas cast uses isinstance(self, U).
For extracting a Rust-only type, see extract.
Example: Casting to a specific Python object but not a subtype
use pyo3::prelude::*;
use pyo3::types::{PyBool, PyInt};
Python::attach(|py| {
let b = PyBool::new(py, true);
assert!(b.is_instance_of::<PyBool>());
let any: &Bound<'_, PyAny> = b.as_any();
// `bool` is a subtype of `int`, so `cast` will accept a `bool` as an `int`
// but `cast_exact` will not.
assert!(any.cast::<PyInt>().is_ok());
assert!(any.cast_exact::<PyInt>().is_err());
assert!(any.cast_exact::<PyBool>().is_ok());
});Sourcepub fn cast_into_exact<U>(self) -> Result<Bound<'py, U>, CastIntoError<'py>>where
U: PyTypeInfo,
pub fn cast_into_exact<U>(self) -> Result<Bound<'py, U>, CastIntoError<'py>>where
U: PyTypeInfo,
Like cast_exact but takes ownership of self.
Sourcepub unsafe fn cast_unchecked<U>(&self) -> &Bound<'py, U>
pub unsafe fn cast_unchecked<U>(&self) -> &Bound<'py, U>
Converts this to a concrete Python type without checking validity.
Safety
Callers must ensure that the type is valid or risk type confusion.
Sourcepub unsafe fn cast_into_unchecked<U>(self) -> Bound<'py, U>
pub unsafe fn cast_into_unchecked<U>(self) -> Bound<'py, U>
Like cast_unchecked but takes ownership of self.
Safety
Callers must ensure that the type is valid or risk type confusion.
Sourceimpl<'py> Bound<'py, PyAny>
impl<'py> Bound<'py, PyAny>
Sourcepub unsafe fn from_owned_ptr(py: Python<'py>, ptr: *mut PyObject) -> Self
pub unsafe fn from_owned_ptr(py: Python<'py>, ptr: *mut PyObject) -> Self
Sourcepub unsafe fn from_owned_ptr_or_opt(
py: Python<'py>,
ptr: *mut PyObject,
) -> Option<Self>
pub unsafe fn from_owned_ptr_or_opt( py: Python<'py>, ptr: *mut PyObject, ) -> Option<Self>
Constructs a new Bound<'py, PyAny> from a pointer. Returns None if ptr is null.
Safety
ptrmust be a valid pointer to a Python object, or nullptrmust be an owned Python reference, as theBound<'py, PyAny>will assume ownership
Sourcepub unsafe fn from_owned_ptr_or_err(
py: Python<'py>,
ptr: *mut PyObject,
) -> PyResult<Self>
pub unsafe fn from_owned_ptr_or_err( py: Python<'py>, ptr: *mut PyObject, ) -> PyResult<Self>
Constructs a new Bound<'py, PyAny> from a pointer. Returns an Err by calling PyErr::fetch
if ptr is null.
Safety
ptrmust be a valid pointer to a Python object, or nullptrmust be an owned Python reference, as theBound<'py, PyAny>will assume ownership
Sourcepub(crate) unsafe fn from_owned_ptr_unchecked(
py: Python<'py>,
ptr: *mut PyObject,
) -> Self
pub(crate) unsafe fn from_owned_ptr_unchecked( py: Python<'py>, ptr: *mut PyObject, ) -> Self
Constructs a new Bound<'py, PyAny> from a pointer without checking for null.
Safety
ptrmust be a valid pointer to a Python objectptrmust be a strong/owned reference
Sourcepub unsafe fn from_borrowed_ptr(py: Python<'py>, ptr: *mut PyObject) -> Self
pub unsafe fn from_borrowed_ptr(py: Python<'py>, ptr: *mut PyObject) -> Self
Sourcepub unsafe fn from_borrowed_ptr_or_opt(
py: Python<'py>,
ptr: *mut PyObject,
) -> Option<Self>
pub unsafe fn from_borrowed_ptr_or_opt( py: Python<'py>, ptr: *mut PyObject, ) -> Option<Self>
Constructs a new Bound<'py, PyAny> from a pointer by creating a new Python reference.
Returns None if ptr is null.
Safety
ptrmust be a valid pointer to a Python object, or null
Sourcepub unsafe fn from_borrowed_ptr_or_err(
py: Python<'py>,
ptr: *mut PyObject,
) -> PyResult<Self>
pub unsafe fn from_borrowed_ptr_or_err( py: Python<'py>, ptr: *mut PyObject, ) -> PyResult<Self>
Constructs a new Bound<'py, PyAny> from a pointer by creating a new Python reference.
Returns an Err by calling PyErr::fetch if ptr is null.
Safety
ptrmust be a valid pointer to a Python object, or null
Source#[doc(hidden)]pub unsafe fn ref_from_ptr<'a>(
_py: Python<'py>,
ptr: &'a *mut PyObject,
) -> &'a Self
#[doc(hidden)]pub unsafe fn ref_from_ptr<'a>(
_py: Python<'py>,
ptr: &'a *mut PyObject,
) -> &'a Self
This slightly strange method is used to obtain &Bound<PyAny> from a pointer in macro code
where we need to constrain the lifetime 'a safely.
Note that 'py is required to outlive 'a implicitly by the nature of the fact that
&'a Bound<'py> means that Bound<'py> exists for at least the lifetime 'a.
Safety
ptrmust be a valid pointer to a Python object for the lifetime'a. Theptrcan be either a borrowed reference or an owned reference, it does not matter, as this is just&Boundthere will never be any ownership transfer.
Sourcepub(crate) unsafe fn ref_from_ptr_or_opt<'a>(
_py: Python<'py>,
ptr: &'a *mut PyObject,
) -> &'a Option<Self>
pub(crate) unsafe fn ref_from_ptr_or_opt<'a>( _py: Python<'py>, ptr: &'a *mut PyObject, ) -> &'a Option<Self>
Variant of the above which returns None for null pointers.
Safety
ptrmust be a valid pointer to a Python object for the lifetime `’a, or null.
Source#[doc(hidden)]pub unsafe fn ref_from_non_null<'a>(
_py: Python<'py>,
ptr: &'a NonNull<PyObject>,
) -> &'a Self
#[doc(hidden)]pub unsafe fn ref_from_non_null<'a>(
_py: Python<'py>,
ptr: &'a NonNull<PyObject>,
) -> &'a Self
This slightly strange method is used to obtain &Bound<PyAny> from a NonNull in macro
code where we need to constrain the lifetime 'a safely.
Note that 'py is required to outlive 'a implicitly by the nature of the fact that &'a Bound<'py> means that Bound<'py> exists for at least the lifetime 'a.
Safety
ptrmust be a valid pointer to a Python object for the lifetime'a. Theptrcan be either a borrowed reference or an owned reference, it does not matter, as this is just&Boundthere will never be any ownership transfer.
Sourceimpl<'py, T> Bound<'py, T>where
T: PyClass,
impl<'py, T> Bound<'py, T>where
T: PyClass,
Sourcepub fn borrow(&self) -> PyRef<'py, T>
pub fn borrow(&self) -> PyRef<'py, T>
Immutably borrows the value T.
This borrow lasts while the returned PyRef exists.
Multiple immutable borrows can be taken out at the same time.
For frozen classes, the simpler get is available.
Examples
#[pyclass]
struct Foo {
inner: u8,
}
Python::attach(|py| -> PyResult<()> {
let foo: Bound<'_, Foo> = Bound::new(py, Foo { inner: 73 })?;
let inner: &u8 = &foo.borrow().inner;
assert_eq!(*inner, 73);
Ok(())
})?;Panics
Panics if the value is currently mutably borrowed. For a non-panicking variant, use
try_borrow.
Sourcepub fn borrow_mut(&self) -> PyRefMut<'py, T>
pub fn borrow_mut(&self) -> PyRefMut<'py, T>
Mutably borrows the value T.
This borrow lasts while the returned PyRefMut exists.
Examples
#[pyclass]
struct Foo {
inner: u8,
}
Python::attach(|py| -> PyResult<()> {
let foo: Bound<'_, Foo> = Bound::new(py, Foo { inner: 73 })?;
foo.borrow_mut().inner = 35;
assert_eq!(foo.borrow().inner, 35);
Ok(())
})?;Panics
Panics if the value is currently borrowed. For a non-panicking variant, use
try_borrow_mut.
Sourcepub fn try_borrow(&self) -> Result<PyRef<'py, T>, PyBorrowError>
pub fn try_borrow(&self) -> Result<PyRef<'py, T>, PyBorrowError>
Sourcepub fn try_borrow_mut(&self) -> Result<PyRefMut<'py, T>, PyBorrowMutError>
pub fn try_borrow_mut(&self) -> Result<PyRefMut<'py, T>, PyBorrowMutError>
Attempts to mutably borrow the value T, returning an error if the value is currently borrowed.
The borrow lasts while the returned PyRefMut exists.
This is the non-panicking variant of borrow_mut.
Sourcepub fn get(&self) -> &T
pub fn get(&self) -> &T
Provide an immutable borrow of the value T.
This is available if the class is frozen and Sync.
Examples
use core::sync::atomic::{AtomicUsize, Ordering};
#[pyclass(frozen)]
struct FrozenCounter {
value: AtomicUsize,
}
Python::attach(|py| {
let counter = FrozenCounter { value: AtomicUsize::new(0) };
let py_counter = Bound::new(py, counter).unwrap();
py_counter.get().value.fetch_add(1, Ordering::Relaxed);
});Sourcepub fn as_super(&self) -> &Bound<'py, T::BaseType>
pub fn as_super(&self) -> &Bound<'py, T::BaseType>
Upcast this Bound<PyClass> to its base type by reference.
If this type defined an explicit base class in its pyclass declaration
(e.g. #[pyclass(extends = BaseType)]), the returned type will be
&Bound<BaseType>. If an explicit base class was not declared, the
return value will be &Bound<PyAny> (making this method equivalent
to as_any).
This method is particularly useful for calling methods defined in an
extension trait that has been implemented for Bound<BaseType>.
See also the into_super method to upcast by value, and the
PyRef::as_super/PyRefMut::as_super methods for upcasting a pyclass
that has already been borrowed.
Example: Calling a method defined on the Bound base type
use pyo3::prelude::*;
#[pyclass(subclass)]
struct BaseClass;
trait MyClassMethods<'py> {
fn pyrepr(&self) -> PyResult<String>;
}
impl<'py> MyClassMethods<'py> for Bound<'py, BaseClass> {
fn pyrepr(&self) -> PyResult<String> {
self.call_method0("__repr__")?.extract()
}
}
#[pyclass(extends = BaseClass)]
struct SubClass;
Python::attach(|py| {
let initializer = PyClassInitializer::from(BaseClass).add_subclass(SubClass);
let obj = Bound::new(py, initializer).unwrap();
assert!(obj.as_super().pyrepr().is_ok());
})Sourcepub fn into_super(self) -> Bound<'py, T::BaseType>
pub fn into_super(self) -> Bound<'py, T::BaseType>
Upcast this Bound<PyClass> to its base type by value.
If this type defined an explicit base class in its pyclass declaration
(e.g. #[pyclass(extends = BaseType)]), the returned type will be
Bound<BaseType>. If an explicit base class was not declared, the
return value will be Bound<PyAny> (making this method equivalent
to into_any).
This method is particularly useful for calling methods defined in an
extension trait that has been implemented for Bound<BaseType>.
See also the as_super method to upcast by reference, and the
PyRef::into_super/PyRefMut::into_super methods for upcasting a pyclass
that has already been borrowed.
Example: Calling a method defined on the Bound base type
use pyo3::prelude::*;
#[pyclass(subclass)]
struct BaseClass;
trait MyClassMethods<'py> {
fn pyrepr(self) -> PyResult<String>;
}
impl<'py> MyClassMethods<'py> for Bound<'py, BaseClass> {
fn pyrepr(self) -> PyResult<String> {
self.call_method0("__repr__")?.extract()
}
}
#[pyclass(extends = BaseClass)]
struct SubClass;
Python::attach(|py| {
let initializer = PyClassInitializer::from(BaseClass).add_subclass(SubClass);
let obj = Bound::new(py, initializer).unwrap();
assert!(obj.into_super().pyrepr().is_ok());
})pub(crate) fn get_class_object(&self) -> &<T as PyClassImpl>::Layout
Sourceimpl<'py, T> Bound<'py, T>
impl<'py, T> Bound<'py, T>
Sourcepub fn as_ptr(&self) -> *mut PyObject
pub fn as_ptr(&self) -> *mut PyObject
Returns the raw FFI pointer represented by self.
Safety
Callers are responsible for ensuring that the pointer does not outlive self.
The reference is borrowed; callers should not decrease the reference count when they are finished with the pointer.
Sourcepub fn into_ptr(self) -> *mut PyObject
pub fn into_ptr(self) -> *mut PyObject
Returns an owned raw FFI pointer represented by self.
Safety
The reference is owned; when finished the caller should either transfer ownership
of the pointer or decrease the reference count (e.g. with pyo3::ffi::Py_DecRef).
Sourcepub fn into_any(self) -> Bound<'py, PyAny>
pub fn into_any(self) -> Bound<'py, PyAny>
Helper to cast to Bound<'py, PyAny>, transferring ownership.
Sourcepub fn as_borrowed<'a>(&'a self) -> Borrowed<'a, 'py, T>
pub fn as_borrowed<'a>(&'a self) -> Borrowed<'a, 'py, T>
Casts this Bound<T> to a Borrowed<T> smart pointer.
Sourcepub fn unbind(self) -> Py<T>
pub fn unbind(self) -> Py<T>
Removes the connection for this Bound<T> from the Python<'py> token,
allowing it to cross thread boundaries.
Sourcepub fn as_unbound(&self) -> &Py<T>
pub fn as_unbound(&self) -> &Py<T>
Removes the connection for this Bound<T> from the Python<'py> token,
allowing it to cross thread boundaries, without transferring ownership.
Sourceimpl<'py> Bound<'py, PyAny>
impl<'py> Bound<'py, PyAny>
Sourcepub(crate) fn lookup_special<N>(
&self,
attr_name: N,
) -> PyResult<Option<Bound<'py, PyAny>>>where
N: IntoPyObject<'py, Target = PyString>,
pub(crate) fn lookup_special<N>(
&self,
attr_name: N,
) -> PyResult<Option<Bound<'py, PyAny>>>where
N: IntoPyObject<'py, Target = PyString>,
Retrieve an attribute value, skipping the instance dictionary during the lookup but still binding the object to the instance.
This is useful when trying to resolve Python’s “magic” methods like __getitem__, which
are looked up starting from the type object. This returns an Option as it is not
typically a direct error for the special lookup to fail, as magic methods are optional in
many situations in which they might be called.
To avoid repeated temporary allocations of Python strings, the intern! macro can be used
to intern attr_name.
pub(crate) fn _get_refcnt(&self) -> isize
Sourceimpl<'py> Bound<'py, PyIterator>
impl<'py> Bound<'py, PyIterator>
Sourcepub fn send(&self, value: &Bound<'py, PyAny>) -> PyResult<PySendResult<'py>>
Available on Py_3_10 and non-PyPy only.
pub fn send(&self, value: &Bound<'py, PyAny>) -> PyResult<PySendResult<'py>>
Py_3_10 and non-PyPy only.Sends a value into a python generator. This is the equivalent of calling
generator.send(value) in Python. This resumes the generator and continues its execution
until the next yield or return statement. When the generator completes, the (optional)
return value will be returned as PySendResult::Return. All subsequent calls will return
PySendResult::Return(None). The first call to send must be made with None as the
argument to start the generator, failing to do so will raise a TypeError.