pub enum IterNextOutput<T, U> {
    Yield(T),
    Return(U),
}
Output of __next__ which can either yield the next value in the iteration, or return a value to raise StopIteration in Python.

See PyIterProtocol for an example.

Variants



Yield(T)

The value yielded by the iterator.



Return(U)

The StopIteration object.