Struct pyo3::PyByteArray [−][src]
pub struct PyByteArray(_);
Represents a Python bytearray.
Methods
impl PyByteArray[src]
impl PyByteArraypub fn new<'p>(py: Python<'p>, src: &[u8]) -> &'p PyByteArray[src]
pub fn new<'p>(py: Python<'p>, src: &[u8]) -> &'p PyByteArrayCreates a new Python bytearray object.
The byte string is initialized by copying the data from the &[u8].
Panics if out of memory.
pub fn from<'p, I>(py: Python<'p>, src: &'p I) -> PyResult<&'p PyByteArray> where
I: ToPyPointer, [src]
pub fn from<'p, I>(py: Python<'p>, src: &'p I) -> PyResult<&'p PyByteArray> where
I: ToPyPointer, Creates a new Python bytearray object from other PyObject, that implements the buffer protocol.
pub fn len(&self) -> usize[src]
pub fn len(&self) -> usizeGets the length of the bytearray.
pub fn is_empty(&self) -> bool[src]
pub fn is_empty(&self) -> boolCheck if bytearray is empty.
pub fn data(&self) -> &mut [u8][src]
pub fn data(&self) -> &mut [u8]Gets the Python bytearray data as byte slice.
pub fn resize(&self, len: usize) -> PyResult<()>[src]
pub fn resize(&self, len: usize) -> PyResult<()>Resize bytearray object.