#[repr(transparent)]pub struct PyAny(UnsafeCell<PyObject>);Expand description
Represents any Python object.
Values of this type are accessed via PyO3’s smart pointers, e.g. as
Py<PyAny> or Bound<'py, PyAny>.
For APIs available on all Python objects, see the PyAnyMethods trait which is implemented for
Bound<'py, PyAny>.
See the guide for an explanation of the different Python object types.
Tuple Fields
0: UnsafeCell<PyObject>Implementations
Sourceimpl PyAny
impl PyAny
#[doc(hidden)]pub const _PYO3_DEF: AddTypeToModule<Self>
#[doc(hidden)]pub const _PYO3_INTROSPECTION_ID: &'static str = "Some(\"builtins\")PyAny"