pub enum IterNextOutput<T, U> {
Yield(T),
Return(U),
}
Output of __next__ which can either yield the next value in the iteration, or
return a value to raise StopIteration in Python.
See PyIterProtocol for an example.
The value yielded by the iterator.
The StopIteration object.
Immutably borrows from an owned value.
Read more
Mutably borrows from an owned value.
Read more
Returns the argument unchanged.
Calls U::from(self).
That is, this conversion is whatever the implementation of
From<T> for U chooses to do.
The type returned in the event of a conversion error.
Performs the conversion.
The type returned in the event of a conversion error.
Performs the conversion.