pub unsafe trait PyLayout<T> { }Expand description
T: PyLayout<U> represents that T is a concrete representation of U in the Python heap.
E.g., PyClassObject is a concrete representation of all pyclasses, and ffi::PyObject
is of PyAny.
This trait is intended to be used internally.
Safety
This trait must only be implemented for types which represent valid layouts of Python objects.
Implementors
impl PyLayout<PyArithmeticError> for PyBaseExceptionObject
impl PyLayout<PyAssertionError> for PyBaseExceptionObject
impl PyLayout<PyAttributeError> for PyBaseExceptionObject
impl PyLayout<PyBaseException> for PyBaseExceptionObject
impl PyLayout<PyBaseExceptionGroup> for PyBaseExceptionObject
impl PyLayout<PyBlockingIOError> for PyBaseExceptionObject
impl PyLayout<PyBrokenPipeError> for PyBaseExceptionObject
impl PyLayout<PyBufferError> for PyBaseExceptionObject
impl PyLayout<PyBytesWarning> for PyBaseExceptionObject
impl PyLayout<PyChildProcessError> for PyBaseExceptionObject
impl PyLayout<PyConnectionAbortedError> for PyBaseExceptionObject
impl PyLayout<PyConnectionError> for PyBaseExceptionObject
impl PyLayout<PyConnectionRefusedError> for PyBaseExceptionObject
impl PyLayout<PyConnectionResetError> for PyBaseExceptionObject
impl PyLayout<PyDeprecationWarning> for PyBaseExceptionObject
impl PyLayout<PyEOFError> for PyBaseExceptionObject
impl PyLayout<PyEncodingWarning> for PyBaseExceptionObject
impl PyLayout<PyEnvironmentError> for PyBaseExceptionObject
impl PyLayout<PyException> for PyBaseExceptionObject
impl PyLayout<PyFileExistsError> for PyBaseExceptionObject
impl PyLayout<PyFileNotFoundError> for PyBaseExceptionObject
impl PyLayout<PyFloatingPointError> for PyBaseExceptionObject
impl PyLayout<PyFutureWarning> for PyBaseExceptionObject
impl PyLayout<PyGeneratorExit> for PyBaseExceptionObject
impl PyLayout<PyIOError> for PyBaseExceptionObject
impl PyLayout<PyImportError> for PyBaseExceptionObject
impl PyLayout<PyImportWarning> for PyBaseExceptionObject
impl PyLayout<PyIndexError> for PyBaseExceptionObject
impl PyLayout<PyInterruptedError> for PyBaseExceptionObject
impl PyLayout<PyIsADirectoryError> for PyBaseExceptionObject
impl PyLayout<PyKeyError> for PyBaseExceptionObject
impl PyLayout<PyKeyboardInterrupt> for PyBaseExceptionObject
impl PyLayout<PyLookupError> for PyBaseExceptionObject
impl PyLayout<PyMemoryError> for PyBaseExceptionObject
impl PyLayout<PyModuleNotFoundError> for PyBaseExceptionObject
impl PyLayout<PyNameError> for PyBaseExceptionObject
impl PyLayout<PyNotADirectoryError> for PyBaseExceptionObject
impl PyLayout<PyNotImplementedError> for PyBaseExceptionObject
impl PyLayout<PyOSError> for PyOSErrorObject
impl PyLayout<PyOverflowError> for PyBaseExceptionObject
impl PyLayout<PyPendingDeprecationWarning> for PyBaseExceptionObject
impl PyLayout<PyPermissionError> for PyBaseExceptionObject
impl PyLayout<PyProcessLookupError> for PyBaseExceptionObject
impl PyLayout<PyRecursionError> for PyBaseExceptionObject
impl PyLayout<PyReferenceError> for PyBaseExceptionObject
impl PyLayout<PyResourceWarning> for PyBaseExceptionObject
impl PyLayout<PyRuntimeError> for PyBaseExceptionObject
impl PyLayout<PyRuntimeWarning> for PyBaseExceptionObject
impl PyLayout<PyStopAsyncIteration> for PyBaseExceptionObject
impl PyLayout<PyStopIteration> for PyStopIterationObject
impl PyLayout<PySyntaxError> for PySyntaxErrorObject
impl PyLayout<PySyntaxWarning> for PyBaseExceptionObject
impl PyLayout<PySystemError> for PyBaseExceptionObject
impl PyLayout<PySystemExit> for PySystemExitObject
impl PyLayout<PyTimeoutError> for PyBaseExceptionObject
impl PyLayout<PyTypeError> for PyBaseExceptionObject
impl PyLayout<PyUnboundLocalError> for PyBaseExceptionObject
impl PyLayout<PyUnicodeDecodeError> for PyBaseExceptionObject
impl PyLayout<PyUnicodeEncodeError> for PyBaseExceptionObject
impl PyLayout<PyUnicodeError> for PyUnicodeErrorObject
impl PyLayout<PyUnicodeTranslateError> for PyBaseExceptionObject
impl PyLayout<PyUnicodeWarning> for PyBaseExceptionObject
impl PyLayout<PyUserWarning> for PyBaseExceptionObject
impl PyLayout<PyValueError> for PyBaseExceptionObject
impl PyLayout<PyWarning> for PyBaseExceptionObject
impl PyLayout<PyZeroDivisionError> for PyBaseExceptionObject
impl PyLayout<PyAny> for PyObject
impl PyLayout<PyBool> for PyObject
impl PyLayout<PyComplex> for PyComplexObject
impl PyLayout<PyDate> for PyDateTime_Date
impl PyLayout<PyDateTime> for PyDateTime_DateTime
impl PyLayout<PyDelta> for PyDateTime_Delta
impl PyLayout<PyDict> for PyDictObject
impl PyLayout<PyFloat> for PyFloatObject
impl PyLayout<PyFrozenSet> for PySetObject
impl PyLayout<PyGenericAlias> for PyDictObject
Available on
Py_3_9 only.