Trait pyo3::type_object::PySizedLayout [−][src]
T: PySizedLayout<U> represents T is not a instance of
PyVarObject.
, in addition that T is a concrete representaion of U.
T: PySizedLayout<U> represents T is not a instance of
PyVarObject.
, in addition that T is a concrete representaion of U.