pub struct PyCFunction(/* private fields */);Expand description
Represents a builtin Python function object.
Values of this type are accessed via PyO3’s smart pointers, e.g. as
Py<PyCFunction> or Bound<'py, PyCFunction>.
Implementations
Sourceimpl PyCFunction
impl PyCFunction
Sourcepub fn new_with_keywords<'py>(
py: Python<'py>,
fun: PyCFunctionWithKeywords,
name: &'static CStr,
doc: &'static CStr,
module: Option<&Bound<'py, PyModule>>,
) -> PyResult<Bound<'py, Self>>
pub fn new_with_keywords<'py>( py: Python<'py>, fun: PyCFunctionWithKeywords, name: &'static CStr, doc: &'static CStr, module: Option<&Bound<'py, PyModule>>, ) -> PyResult<Bound<'py, Self>>
Create a new built-in function with keywords (*args and/or **kwargs).
To create name and doc static strings on Rust versions older than 1.77 (which added c““ literals),
use the c_str! macro.
Sourcepub fn new<'py>(
py: Python<'py>,
fun: PyCFunction,
name: &'static CStr,
doc: &'static CStr,
module: Option<&Bound<'py, PyModule>>,
) -> PyResult<Bound<'py, Self>>
pub fn new<'py>( py: Python<'py>, fun: PyCFunction, name: &'static CStr, doc: &'static CStr, module: Option<&Bound<'py, PyModule>>, ) -> PyResult<Bound<'py, Self>>
Create a new built-in function which takes no arguments.
To create name and doc static strings on Rust versions older than 1.77 (which added c““ literals),
use the c_str! macro.
Sourcepub fn new_closure<'py, F, R>(
py: Python<'py>,
name: Option<&'static CStr>,
doc: Option<&'static CStr>,
closure: F,
) -> PyResult<Bound<'py, Self>>
pub fn new_closure<'py, F, R>( py: Python<'py>, name: Option<&'static CStr>, doc: Option<&'static CStr>, closure: F, ) -> PyResult<Bound<'py, Self>>
Create a new function from a closure.
Examples
Python::attach(|py| {
let add_one = |args: &Bound<'_, PyTuple>, _kwargs: Option<&Bound<'_, PyDict>>| -> PyResult<_> {
let i = args.extract::<(i64,)>()?.0;
Ok(i+1)
};
let add_one = PyCFunction::new_closure(py, None, None, add_one).unwrap();
py_run!(py, add_one, "assert add_one(42) == 43");
});Trait Implementations
Sourceimpl PyTypeInfo for PyCFunction
impl PyTypeInfo for PyCFunction
Sourceconst NAME: &'static str = "PyCFunction"
const NAME: &'static str = "PyCFunction"
prefer using ::type_object(py).name() to get the correct runtime value
Sourceconst MODULE: Option<&'static str>
const MODULE: Option<&'static str>
prefer using ::type_object(py).module() to get the correct runtime value
Sourceconst TYPE_HINT: PyStaticExpr
const TYPE_HINT: PyStaticExpr
Sourcefn type_object_raw(py: Python<'_>) -> *mut PyTypeObject
fn type_object_raw(py: Python<'_>) -> *mut PyTypeObject
Sourcefn is_type_of(obj: &Bound<'_, PyAny>) -> bool
fn is_type_of(obj: &Bound<'_, PyAny>) -> bool
object is an instance of this type or a subclass of this type.impl DerefToPyAny for PyCFunction